<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class FmtSolicitacaoServFactory
{
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getSolicitacoesServicos()
    {
                    $sql = "SELECT 
                        SOLSERV.NRSOLICITACAOSERV, 
                        TO_CHAR(SOLSERV.DTSOLICITACAOSERV, 'DD/MM/YY') AS DTSOLICITACAOSERV, 
                        CATMANUT.NMCATEGORIAMANUT, 
                        EQUIP.NMEQUIPAMENTO,
                        SOLSERV.DSSOLICITACAOSERV, 
                        NVL(OPER.NMOPERADOR, '') AS NMOPERADOR,
                        SOLSERV.CDOPERSOLICITANTE, 
                        SOLSERV.IDSITUACAO,
                        SOLSERV.NREQUIPAMENTO, 
                        SOLSERV.NRCATEGORIAMANUT, 
                        FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK,    
                        FCM_CHECKLISTAGEND.NMCHECKLISTAGEND,
                        SOLSERV.NRLOCALCLIENTE,
                        FCM_LOCALCLI.NMLOCALCLIENTE,
                        FCM_LOCALCLI.CDCLIENTE,
                        CLIENTE.NMFANTCLIE,
                        TO_CHAR(
                            COALESCE(
                                FCM_GERENCIAMENTOCHK.DTHORAAGEND, 
                                FCM_CHECKLISTAGEND.DTHORAAGEND
                            ), 
                            'DD/MM/YY HH24:MI'
                        ) AS DTHORAAGEND,
                        SOLSERV.NRIMPACTOMANUT, 
                        IMPMANUT.NMIMPACTOMANUT, 
                        SOLSERV.NRPRIORIDADEMANUT, 
                        SOLSERV.NMSOLICITACAOSERV, 
                        SOLSERV.NRGERENCIAMENTOCHK, 
                        SOLSERV.NRCHECKLISTAGEND,
                        (
                            SELECT 
                                LISTAGG(FCM_IMGSOLICITACAOSERV.DSCAMINHO, ', ') 
                                WITHIN GROUP (ORDER BY FCM_IMGSOLICITACAOSERV.DSCAMINHO)
                            FROM 
                                FCM_IMGSOLICITACAOSERV
                            WHERE 
                                SOLSERV.NRSOLICITACAOSERV = FCM_IMGSOLICITACAOSERV.NRSOLICITACAOSERV
                        ) AS DSCAMINHOABC,
                        (
                            SELECT 
                                LISTAGG(ESCMANUT.NRESCOPOMANUT, ', ') 
                                WITHIN GROUP (ORDER BY ESCMANUT.NRESCOPOMANUT)
                            FROM 
                                FCM_RELESCSOLICITASERV RELESCOSOL
                            JOIN 
                                FCM_ESCOPOMANUT ESCMANUT
                                ON RELESCOSOL.NRESCOPOMANUT = ESCMANUT.NRESCOPOMANUT
                            WHERE 
                                SOLSERV.NRSOLICITACAOSERV = RELESCOSOL.NRSOLICITACAOSERV
                        ) AS NRESCOPOMANUT,
                        (
                            SELECT 
                                LISTAGG(ESCMANUT.NMESCOPOMANUT, ', ') 
                                WITHIN GROUP (ORDER BY ESCMANUT.NRESCOPOMANUT)
                            FROM 
                                FCM_RELESCSOLICITASERV RELESCOSOL
                            JOIN 
                                FCM_ESCOPOMANUT ESCMANUT
                                ON RELESCOSOL.NRESCOPOMANUT = ESCMANUT.NRESCOPOMANUT
                            WHERE 
                                SOLSERV.NRSOLICITACAOSERV = RELESCOSOL.NRSOLICITACAOSERV
                        ) AS NMESCOPOMANUT,
                        CASE SOLSERV.IDSITUACAO
                            WHEN '1' THEN 'Pendente'
                            WHEN '2' THEN 'Em Andamento'
                            WHEN '3' THEN 'Aguardando vistoria'
                            WHEN '4' THEN 'Concluída'
                            WHEN '5' THEN 'Cancelada'
                            ELSE NULL
                        END AS SITUACAO
                    FROM 
                        FCM_SOLICITACAOSERV SOLSERV
                    LEFT JOIN 
                        FCM_EQUIPAMENTO EQUIP
                        ON SOLSERV.NREQUIPAMENTO = EQUIP.NREQUIPAMENTO
                    LEFT JOIN 
                        FCM_CATEGORIAMANUT CATMANUT
                        ON SOLSERV.NRCATEGORIAMANUT = CATMANUT.NRCATEGORIAMANUT
                    LEFT JOIN 
                        OPERADOR OPER
                        ON SOLSERV.CDOPERSOLICITANTE = OPER.CDOPERADOR
                    LEFT JOIN 
                        FCM_IMPACTOMANUT IMPMANUT
                        ON SOLSERV.NRIMPACTOMANUT = IMPMANUT.NRIMPACTOMANUT
                    LEFT JOIN 
                        FCM_GERENCIAMENTOCHK
                        ON SOLSERV.NRGERENCIAMENTOCHK = FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK
                    LEFT JOIN 
                        FCM_CHECKLISTAGEND
                        ON SOLSERV.NRCHECKLISTAGEND = FCM_CHECKLISTAGEND.NRCHECKLISTAGEND
                    LEFT JOIN FCM_LOCALCLI
                        ON FCM_LOCALCLI.NRLOCALCLIENTE = SOLSERV.NRLOCALCLIENTE
                    LEFT JOIN CLIENTE
                        ON FCM_LOCALCLI.CDCLIENTE = CLIENTE.CDCLIENTE
                        ORDER BY SOLSERV.DTSOLICITACAOSERV DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function recusarSolicitacao($parametros)
    {
        $sql = "UPDATE FCM_SOLICITACAOSERV
                SET IDSITUACAO = '4'
                WHERE NRSOLICITACAOSERV = :NRSOLICITACAOSERV";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertComentarioSolicitacao($parametros)
    {
        $sql = "INSERT INTO FCM_COMENTSOLICITSERV (
                    NRCOMENTSOLICITSERV,    NRSOLICITACAOSERV,  DTCOMENTARIO,       DSCOMENTARIO,
                    NRORG,                  DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,               CDOPERULTATU,       NRORGULTATU,        CDOPERCOMENTARIO
                )
                VALUES (
                    :NRCOMENTSOLICITSERV,   :NRSOLICITACAOSERV, SYSDATE,            :DSCOMENTARIO,
                    :NRORG,                 SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,                :CDOPERULTATU,      :NRORGULTATU,       :CDOPERCOMENTARIO
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function saveSolicitacaoServico($parametros)
    {
        $sql = "INSERT INTO FCM_SOLICITACAOSERV (
                    NRSOLICITACAOSERV,  DTSOLICITACAOSERV,  NREQUIPAMENTO,      NRCATEGORIAMANUT,
                    NRIMPACTOMANUT,     DSSOLICITACAOSERV,  NRORG,              IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,
                    CDOPERULTATU,       NRORGULTATU,        NMSOLICITACAOSERV,  CDOPERSOLICITANTE,
                    NRGERENCIAMENTOCHK, NRCHECKLISTAGEND, NRLOCALCLIENTE
                )
                VALUES (
                    :NRSOLICITACAOSERV, SYSDATE,            :NREQUIPAMENTO,     :NRCATEGORIAMANUT,
                    :NRIMPACTOMANUT,    :DSSOLICITACAOSERV, :NRORG,             'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,     SYSDATE,
                    :CDOPERULTATU,      :NRORGULTATU,       :NMSOLICITACAOSERV, :CDOPERSOLICITANTE,
                    :NRGERENCIAMENTOCHK, :NRCHECKLISTAGEND, :NRLOCALCLIENTE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertFotoSolicitacao($parametros)
    {
        $sql = "INSERT INTO FCM_IMGSOLICITACAOSERV (
                    NRIMGSOLICITACAOSERV,   NRSOLICITACAOSERV,  DSCAMINHO,  NRORG,
                    DTINCLUSAO,             DSTYPE,             NRSIZE
                )
                VALUES (
                    :NRIMGSOLICITACAOSERV,  :NRSOLICITACAOSERV, :DSCAMINHO, :NRORG,
                    SYSDATE,                :DSTYPE,            :NRSIZE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getImpactoManut()
    {
        $sql = "SELECT NRIMPACTOMANUT, NMIMPACTOMANUT
                FROM FCM_IMPACTOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getPrioridadesManut()
    {
        $sql = "SELECT NRPRIORIDADEMANUT, NMPRIORIDADEMANUT
                FROM FCM_PRIORIDADEMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosSolicitacao($parametros)
    {
        $sql = "SELECT NRIMGSOLICITACAOSERV, DSCAMINHO
                FROM FCM_IMGSOLICITACAOSERV
                WHERE NRSOLICITACAOSERV= :NRSOLICITACAOSERV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getComentariosSolicitacao($parametros)
    {
        $sql = "SELECT COMENT.NRCOMENTSOLICITSERV, NVL(OPER.NMOPERADOR, 'Anônimo') AS NMOPERADOR, TO_CHAR(COMENT.DTINCLUSAO, 'DD/MM/YYYY') AS DTINCLUSAO, COMENT.DSCOMENTARIO
                FROM FCM_COMENTSOLICITSERV COMENT
                LEFT JOIN OPERADOR OPER
                ON COMENT.CDOPERCOMENTARIO = OPER.CDOPERADOR
                WHERE COMENT.NRSOLICITACAOSERV = :NRSOLICITACAOSERV
                ORDER BY DTINCLUSAO, NRCOMENTSOLICITSERV DESC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getOperadores()
    {
        $sql = "SELECT CDOPERADOR, NMOPERADOR
                FROM OPERADOR
                WHERE IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFuncionarios()
    {
        $sql = "SELECT NRFUNCIONARIO, NMFUNCIONARIO
                FROM FCM_FUNCIONARIO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFuncionarioLogado($parametros)
    {
        $sql = "SELECT NRFUNCIONARIO
                FROM FCM_FUNCIONARIO
                WHERE CDOPERADOR = :CDOPERADOR";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function insertHistoricoVistoria($parametros)
    {
        $sql = "INSERT INTO FCM_GERCHKHISTORICO (
                    NRGERCHKHISTORICO,      NRGERENCIAMENTOCHK, NMGERENCIAMENTOCHK, DSGERENCIAMENTOCHK,
                    DTHISTGERENCIAMENTOCHK, IDTIPOHISTORICO,    IDATIVO,            NRORG,
                    DTINCLUSAO,             CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,
                    CDOPERULTATU,           NRORGULTATU
                )
                VALUES (
                    :NRGERCHKHISTORICO,     :NRGERENCIAMENTOCHK, :NMGERENCIAMENTOCHK, :DSGERENCIAMENTOCHK,
                    SYSDATE,                :IDTIPOHISTORICO,    'S',                 :NRORG,
                    SYSDATE,                :CDOPERINCLUSAO,     :NRORGINCLUSAO,      SYSDATE,
                    :CDOPERULTATU,          :NRORGULTATU
                )";

        return $this->connection->executeStatement($sql, $parametros);
    }

    public function insertHistoricoTarefa($parametros)
    {
        $sql = "INSERT INTO FCM_CHKAGENDHISTORICO (
                    NRCHKAGENDHISTORICO,    NRCHECKLISTAGEND,   NMCHECKLISTAGEND,   DSCHKAGENDHISTORICO,
                    DTCHKAGENDHISTORICO,    IDTIPOHISTORICO,    IDATIVO,            NRORG,
                    DTINCLUSAO,             CDOPERINCLUSAO,     NRORGINCLUSAO,      DTULTATU,
                    CDOPERULTATU,           NRORGULTATU
                )
                VALUES (
                    :NRCHKAGENDHISTORICO,   :NRCHECKLISTAGEND,  :NMCHECKLISTAGEND,  :DSCHKAGENDHISTORICO,
                    SYSDATE,                :IDTIPOHISTORICO,   'S',                :NRORG,
                    SYSDATE,                :CDOPERINCLUSAO,    :NRORGINCLUSAO,     SYSDATE,
                    :CDOPERULTATU,          :NRORGULTATU
                )";

        return $this->connection->executeStatement($sql, $parametros);
    }

    public function insertHistorico($parametros)
    {
        $sql = "INSERT INTO FCM_ORDEMSERVHISTORICO(
                    NRORDEMSERVHISTORICO,   NRSOLICITACAOSERV,  NRORDEMSERV, 
                    DTHISTORDEMSERV,        IDATIVO,            DSORDEMSERVHISTORICO, 
                    NRORG,                  DTINCLUSAO,         CDOPERINCLUSAO, 
                    NRORGINCLUSAO
                )
                VALUES(
                    :NRORDEMSERVHISTORICO,   :NRSOLICITACAOSERV, :NRORDEMSERV, 
                    SYSDATE,                 'S',                :DSORDEMSERVHISTORICO, 
                    :NRORG,                   SYSDATE,           :CDOPERINCLUSAO, 
                    :NRORGINCLUSAO
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getNmchecklist($parametros)
    {
        $sql = "SELECT NMCHECKLISTAGEND 
                FROM FCM_CHECKLISTAGEND
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";
    
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getNmgerenciamento($parametros)
    {
        $sql = "SELECT NMGERENCIAMENTOCHK 
                FROM FCM_GERENCIAMENTOCHK
                WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK";
    
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getHistoricoSolServ($parametros) 
    {
        $sql = "SELECT FCM_ORDEMSERVHISTORICO.NRORDEMSERVHISTORICO, FCM_ORDEMSERVHISTORICO.NRSOLICITACAOSERV, FCM_ORDEMSERVHISTORICO.NRORDEMSERV, 
                FCM_ORDEMSERVHISTORICO.DSORDEMSERVHISTORICO, FCM_ORDEMSERVHISTORICO.DTHISTORDEMSERV, OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR
                FROM FCM_ORDEMSERVHISTORICO
                JOIN OPERADOR ON OPERADOR.CDOPERADOR = FCM_ORDEMSERVHISTORICO.CDOPERINCLUSAO
                WHERE NRSOLICITACAOSERV = :NRSOLICITACAOSERV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getTecnicos($locais)
    {
        $sql = "SELECT DISTINCT OPERADOR.CDOPERADOR, OPERADOR.NMOPERADOR
                    FROM OPERADOR
                    JOIN FCM_FUNCIONARIOSHIERARQ
                    ON FCM_FUNCIONARIOSHIERARQ.CDOPERADOR = OPERADOR.CDOPERADOR
                    JOIN FCM_HIERARQUIAFUNC
                    ON FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC = FCM_FUNCIONARIOSHIERARQ.NRHIERARQUIAFUNC
                LEFT JOIN FCM_RELACHABOPER 
                    ON FCM_RELACHABOPER.CDOPERADOR = OPERADOR.CDOPERADOR
                LEFT JOIN  FCM_RELACHABATIV
                    ON  FCM_RELACHABATIV.NRHABILIDADE = FCM_RELACHABOPER.NRHABILIDADE
                    LEFT JOIN FCM_TAREFACHK
                    ON FCM_TAREFACHK.NRTAREFA = FCM_RELACHABATIV.NRTAREFA
                WHERE FCM_HIERARQUIAFUNC.NRLOCALCLIENTE IN (SELECT NRLOCALCLIENTE
                                                                FROM FCM_LOCALCLI
                                                            START WITH NRLOCALCLIENTE IN ($locais)
                                                            CONNECT BY PRIOR NRLOCALCLIENTEPAI = NRLOCALCLIENTE)";


        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getEquipeTecnicos($locais)
    {
        $sql = "SELECT DISTINCT FCM_EQUIPE.NREQUIPE, FCM_EQUIPE.NMEQUIPE 
                    FROM FCM_EQUIPE
                    JOIN FCM_RELACEQUIPEOPER 
                    ON FCM_RELACEQUIPEOPER.NREQUIPE = FCM_EQUIPE.NREQUIPE
                    JOIN FCM_FUNCIONARIOSHIERARQ
                    ON FCM_FUNCIONARIOSHIERARQ.CDOPERADOR = FCM_RELACEQUIPEOPER.CDOPERADOR
                    JOIN FCM_HIERARQUIAFUNC
                    ON FCM_HIERARQUIAFUNC.NRHIERARQUIAFUNC = FCM_FUNCIONARIOSHIERARQ.NRHIERARQUIAFUNC
                    WHERE FCM_HIERARQUIAFUNC.NRLOCALCLIENTE IN (SELECT NRLOCALCLIENTE
                                                                FROM FCM_LOCALCLI
                                                                START WITH NRLOCALCLIENTE IN ($locais)
                                                                CONNECT BY PRIOR NRLOCALCLIENTEPAI = NRLOCALCLIENTE)
                    AND FCM_EQUIPE.IDATIVO = 'S'";  // Condição adicionada para trazer apenas as equipes ativas

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}
