import { Utilities } from '@/util/Utilities';
import {
    Button, DialogService, Form, LoadingService, Modal, ModalService, Select, SelectMultiple, Text, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekTreeGrid, TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00003000ChecklistController {
    public modalChecklist!: Modal;

    public edicaoChecklist!: Modal;

    public currentRow: any;

    public dados: any;

    public dadosChecklist: any = {};

    public resposta: any;

    public validaEdicao: any;

    public nrEdicao: any;

    public visibilidadeGrid: boolean = false;

    public visibilidadeBotaoGerar: boolean = !this.visibilidadeGrid;

    public check: boolean = false;

    public modalErroInativar!: Modal;

    public async criarModais() {
        this.modalChecklist = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00003_000_checklist/modal/fm_00003_001_salvar_checklist' });
        this.edicaoChecklist = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00003_000_checklist/modal/fm_00003_004_editar_checklist' });
        this.modalErroInativar = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00003_000_checklist/modal/fm_00003_008_modal_inatividade_checklist' });
    }

    public destruirModais() {
        this.modalChecklist?.destroy();
        this.edicaoChecklist?.destroy();
        this.modalErroInativar?.destroy();
    }

    public async abrirModal() {
        this.modalChecklist.show();
    }

    public fecharModal() {
        this.modalChecklist.hide();
    }

    public async abrirModalErroInatividade() {
        this.modalErroInativar.show();
    }

    public fecharModalInatividade() {
        this.modalErroInativar.hide();
    }

    public gerarDadosChecklist() {
        const form = Metadata.getInstance<Form>('form-checklist');
        const treeGrid = Metadata.getInstance<TekTreeGrid>('tree-grid-checklist');
        const selectAtividades = Metadata.getInstance<SelectMultiple>('atividades-checklist');
        const selectItens = Metadata.getInstance<SelectMultiple>('itens-checklist');

        if (Utilities.validateForm(form)) {
            this.visibilidadeGrid = true;
            this.visibilidadeBotaoGerar = !this.visibilidadeGrid;
            treeGrid.datasource.setFilter({
                NRTAREFA: selectAtividades.value,
                NRATIVIDADE: selectItens.value,
            });
        }
    }

    public changeAtividades(atividades: any) {
        const selectAtividades = Metadata.getInstance<Select>('atividades-checklist');
        if (atividades.length > 0) {
            selectAtividades.disabled = false;
            selectAtividades.datasource.setFilter({
                NRATIVIDADE: atividades,
            });
        } else {
            selectAtividades.disabled = true;
            selectAtividades.value = '';
            this.visibilidadeGrid = false;
        }
    }

    public changeVisibilidadeGrid({ component }: any) {
        if (component.value.length > 0) {
            this.visibilidadeGrid = false;
        }
    }

    public habilitarBotaoSalvar() {
        Metadata.getInstance<Button>('salvar-checklist').disabled = true;
    }

    public async salvarChecklist() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-checklist');
        const tekGrid = Metadata.getInstance<TekTreeGrid>('tek-grid-checklist');

        if (grid.selectedRows.length > 0) {
            const escopos = (grid.selectedRows.filter((elemento: any) => elemento.TIPO === 'Atividade')).map((item: any) => ({ NRTAREFA: item.CODIGOAUX, NRESCOPO: item.CODIGOPAIAUX }));
            const criterios = (grid.selectedRows.filter((elemento: any) => elemento.TIPO === 'Critério')).map((item: any) => ({ NRCRITERIO: item.CODIGOAUX, NRTAREFA: item.CODIGOPAIAUX }));

            await Utilities.save('/salvarChecklist', {
                ESCOPOS: escopos,
                CRITERIOS: criterios,
                NMCHECKLIST: Metadata.getInstance<TextInput>('nome-checklist').value,
                NRTIPOLOCALCLI: Metadata.getInstance<Select>('tipo-local-checklist').value,
                NRATIVIDADE: Metadata.getInstance<SelectMultiple>('itens-checklist').value,
                NRTAREFA: Metadata.getInstance<SelectMultiple>('atividades-checklist').value,
                IDATIVO: Metadata.getInstance<Select>('IDATIVO').value,
            }).then((Response) => {
                this.resposta = Response;
                this.fecharModal();
                this.fecharModalEdicao();
                tekGrid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        } else {
            Utilities.alertMessage('Algum escopo deve ser selecionado', 'error');
            this.resposta = 'error';
        }
    }

    // EDIÇÃO MODAL CHECKLIST

    public async abrirModalEdicao({ row }: any) {
        await this.edicaoChecklist.show();
        this.getDadosChecklistEdicao(row);
        this.currentRow = row;
    }

    public fecharModalEdicao() {
        this.edicaoChecklist.hide();
        this.dados = [];
        this.currentRow = [];
    }

    public getDadosChecklistEdicao(row: any) {
        const form = Metadata.getInstance<Form>('form-edicao-checklist');
        const n = Metadata.getInstance<Text>('n-checklist');
        const nome = Metadata.getInstance<TextInput>('nome-edicao-checklist');
        const tipoLocal = Metadata.getInstance<Select>('tipo-local-edicao-checklist');
        const ativo = Metadata.getInstance<Select>('IDATIVO-edicao-checklist');
        const itens = Metadata.getInstance<SelectMultiple>('itens-edicao-checklist');
        const atividades = Metadata.getInstance<SelectMultiple>('atividades-edicao-checklist');
        const arvore = Metadata.getInstance<TekTreeGrid>('tree-grid-edicao-checklist');

        form.clearForm();
        arvore.datasource.data = [];
        (arvore.datasource as MemoryDatasource).allData = [];

        LoadingService.show();
        Http.post('/getDadosChecklistEdicao', {
            NRCHECKLIST: row.NRCHECKLIST,
        }).then((response:IDictionary) => {
            if (response.data.length > 0) {
                const dadosGerais = response.data[0].data[0];

                n.text = dadosGerais.NRCHECKLIST;
                nome.value = dadosGerais.NMCHECKLIST;
                tipoLocal.value = dadosGerais.NRTIPOLOCALCLI;
                ativo.value = dadosGerais.IDATIVO;
                atividades.setValue(dadosGerais.NRTAREFA.map((str: any) => parseInt(str, 10)));
                itens.setValue(dadosGerais.NRATIVIDADE.map((str: any) => parseInt(str, 10)));

                arvore.datasource.updateData(response.data[1].data);
                arvore.datasource.get();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public validaChanges({ component }: any) {
        const selectAtividades = Metadata.getInstance<Select>('atividades-checklist');
        if (!component) {
            selectAtividades.disabled = false;
            selectAtividades.datasource.setFilter({
                NRATIVIDADE: [this.dados],
            });
        } else {
            this.changeAtividades(component.value);
        }
    }

    public async editarChecklist() {
        const form = Metadata.getInstance<Form>('form-edicao-checklist');
        const grid = Metadata.getInstance<Select>('tek-grid-checklist');

        if (Utilities.validateForm(form)) {
            Http.post('/edicaoChecklist', {
                NRCHECKLIST: Metadata.getInstance<Text>('n-checklist').text,
                IDATIVO: Metadata.getInstance<Select>('IDATIVO-edicao-checklist').value,
                NMCHECKLIST: Metadata.getInstance<TextInput>('nome-edicao-checklist').value,
            }).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data);
                }
                grid.datasource.get();
                this.fecharModalEdicao();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public dialogInativarChecklist({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003000ChecklistController.inativarChecklist', 'Excluir Registro', 'Deseja realmente excluir todos os registros selecionados?');
    }

    public inativarChecklist() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-checklist');

        Utilities.deleteInfoGrid('/excluirChecklist', { component: grid }).then((response: any) => {
            DialogService.hide();
            if (response.data.length > 0) {
                this.montarModalErroInatividade(response.data, true);
            }
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public async montarModalErroInatividade(tarefas: any, exclusao = false) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade-checklist');
        const campoTextErro = Metadata.getInstance<Text>('texto-erro-checklist');

        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        if (exclusao) {
            campoTextErro.text = '<strong>Não é possível excluir o(s) checklist(s) selecionado(s) porque há tarefas associada(s) a ele(s). *</strong>';
            grid.getColumn('IDATIVO').isVisible = true;
        } else {
            campoTextErro.text = '<strong>Não é possível inativar o(s) checklist(s) selecionado(s) porque há tarefas ativas associada(s) a ele(s) que ainda nao foram concluídas ou canceladas. *</strong>';
            grid.getColumn('IDATIVO').isVisible = false;
        }

        grid.datasource.updateData(tarefas);
        grid.datasource.get();
    }
}
