import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, Modal, ModalService, Text, TextInput,
} from '@zeedhi/common';
import {
    Http, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00003004EscopoServicoController {
    private modalEscopo!: Modal;

    public controlarEdicao: boolean = false;

    public modalErroInativarEscopo!: Modal;

    public async criarModais() {
        this.modalErroInativarEscopo = await ModalService.createFromJson('fm_00003_000_checklist/modal/fm_00003_005_modal_inatividade_escopo', true);
    }

    public async abrirModalErroInatividade() {
        this.modalErroInativarEscopo.show();
    }

    public fecharModalInatividade() {
        this.modalErroInativarEscopo.hide();
    }

    public async abrirModal() {
        await this.modalEscopo.show();
        this.modalEscopo = await ModalService.createFromJsonPost('', { jsonPath: '/fm_00003_000_checklist/modal/fm_00003_004_salvar_escopo' });
    }

    public fecharModal() {
        this.modalEscopo.hide();
        this.modalEscopo?.destroy();
    }

    public destruirModais() {
        this.modalErroInativarEscopo.destroy();
    }

    public salvarEscopo({ event }: any) {
        event.preventDefault();

        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-servico');

        if (Utilities.validateGrid(grid)) {
            LoadingService.show();
            Http.post('/salvarEscopoServico', grid.getEditedRows()).then((response) => {
                if (response.data.length > 0) {
                    this.montarModalErroInatividade(response.data, false);
                }
                Utilities.showSuccessMessage();
                grid.cancelEditedRows();
                grid.datasource.get();
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public adicionarNovaLinha({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRESCOPOSTAREFA: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMESCOPOSTAREFA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public dialogInativarEscopos({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00003004EscopoServicoController.inativarEscopos', 'Excluir Registros', 'Deseja realmente excluir todos os registros selecionados?');
    }

    public inativarEscopos() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-escopo-servico');
        Utilities.deleteInfoGrid('/excluirEscopos', { component: grid }).then((response: any) => {
            if (response.data.length > 0) {
                this.montarModalErroInatividade(response.data, true);
            }
            grid.datasource.get();
            DialogService.hide();
        });
    }

    public async montarModalErroInatividade(tarefas: any, exclusao = false) {
        await this.abrirModalErroInatividade();
        const grid = Metadata.getInstance<TekGrid>('grid-tarefas-conflito-inatividade-escopo');
        const campoTextErro = Metadata.getInstance<Text>('texto-erro-escopo');

        if (exclusao) {
            campoTextErro.text = '<strong>Não é possível excluir o(s) escopos(s) selecionado(s) porque há iten(s) associado(s) a ele(s). *</strong>';
            grid.getColumn('NMTAREFA').isVisible = false;
        } else {
            campoTextErro.text = '<strong>Não é possível inativar o(s) escopos(s) selecionado(s) porque há iten(s) e atividade(s) ativa(s) associada(s) a ele(s). *</strong>';
            grid.getColumn('NMATIVIDADE').isVisible = true;
            grid.getColumn('NMTAREFA').isVisible = true;
        }

        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRITEMCHECKLIST: element.NRITEMCHECKLIST,
                NRATIVIDADE: element.NRATIVIDADE,
                NMATIVIDADE: element.NMATIVIDADE,
                NRTAREFA: element.NRTAREFA,
                NMTAREFA: element.NMTAREFA,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }
}
