import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple,
    Form, LoadingService, Modal, ModalService, Text,
    Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { IKanbanEventParam, Kanban } from '@zeedhi/zd-kanban-common';

export class FmKanbanVistorias {
    public modalInfos!: Modal;

    public modalMoved!: Modal;

    currentRow: any;

    nrgerenciamentochk: any;

    public visibilidadeColunaAtraso({ component }: any) {
        const situacao = component.name.split('_')[1];
        const atraso = component.name.split('_')[2];

        if (situacao === '1' && atraso !== '  ') {
            component.isVisible = true;
        } else {
            component.isVisible = false;
        }
    }

    public async criarModais() {
        this.modalInfos = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00005_000_vistorias/modal/fm_modal_info_relatorio_vistorias' });
        this.modalMoved = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00005_000_vistorias/modal/fm_modal_mover_task' });
    }

    public destruirModais() {
        this.modalInfos.destroy();
        this.modalMoved.destroy();
    }

    public task: any;

    public openModalMoveTask({ task }: IKanbanEventParam) {
        if (task.columnFrom.id !== task.columnTo.id) {
            this.task = task;
            this.modalMoved.show();
            Metadata.getInstance<Form>('form-justificativa-moved-task').clearForm();
        }
    }

    public closeModalMoveTask(cancel: boolean = true) {
        if (cancel) {
            const kanban = Metadata.getInstance<Kanban>('kanban-vistorias');
            kanban.moveTask(this.task.task, { index: this.task.oldIndex, columnId: this.task.columnFrom.id });
        }
        this.modalMoved.hide();
    }

    /* Visualizar informações Vistoria */
    public async abrirModal({ component }: any) {
        /* eslint-disable */
        this.nrgerenciamentochk = component.name.split('_')[0];
        const idsituacao = component.name.split('_')[1];
        /* eslint-enable */
        if (idsituacao !== 'Tarefa') {
            this.modalInfos.show();
            setTimeout(() => {
                this.montarInformacoesComplementares();
                const tekGrid = Metadata.getInstance<TekGrid>('grid-historico-vistoria');
                tekGrid.datasource.setFilter({
                    NRGERENCIAMENTOCHK: this.nrgerenciamentochk,
                });
            }, 70);
        }
    }

    public closeModal() {
        this.modalInfos.hide();
    }

    public montarInformacoesComplementares() {
        LoadingService.show();
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-naoconformidades');
        const semItens = Metadata.getInstance<Text>('nao-ha-conformidades');

        Http.post('/getCriteriosVistoria', {
            NRGERENCIAMENTOCHK: this.nrgerenciamentochk,
        }).then((response: IDictionary) => {
            const dados = response.data.data;

            checkbox.value = dados.filter((elemento: any) => elemento.IDMARCADO === 'S').map((elemento: any) => elemento.NRCRITERIOTARGERCHK);
            (checkbox.datasource as SimpleDatasource)!.data = dados;

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            // não permite repetição de valores (new set)
            const observacoes = [
                ...new Set(
                    dados.map((element: any) => element.DSOBSTAREFAGERCHK),
                ),
            ];
            Metadata.getInstance<Textarea>('observacoes-vistoria').value = observacoes.join('\n') || 'Não há observações';
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public moveTask() {
        const form = Metadata.getInstance<Form>('form-justificativa-moved-task');
        const kanban = Metadata.getInstance<Kanban>('kanban-vistorias');
        const gridPrincipal = Metadata.getInstance<TekGrid>('tek-grid-vistorias');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/moveVistoria', {
                IDSITUACAOGERCHK: this.task.columnTo.id,
                NRGERENCIAMENTOCHK: this.task.task.NRGERENCIAMENTOCHK,
                NMGERENCIAMENTOCHK: this.task.task.NMGERENCIAMENTOCHK,
                JUSTIFICATIVA: Metadata.getInstance<Textarea>('text-area-justificativa-moved-task').value,
            }).then(() => {
                kanban.datasource.get();
                Utilities.showSuccessMessage();
                this.closeModalMoveTask(false);
                gridPrincipal.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }
}
