import { Utilities } from '@/util/Utilities';
import {
    DialogService, LoadingService, Modal, ModalService, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, MemoryDatasource, Metadata,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class Fm00040001CriteriosAuditoriaController {
    public parametrizacaoPercentual: any = false;

    public parametrizacaoNota: any = false;

    public modalErroExcluirCriterio!: Modal;

    public modalErroinativarCriterio!: Modal;

    constructor() {
        this.criarModais();
    }

    public async criarModais() {
        this.modalErroExcluirCriterio = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_exclusao_criterio', true);
        this.modalErroinativarCriterio = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_inatividade_criterio', true);
    }

    public async abrirModalErro() {
        if (this.modalErroExcluirCriterio) {
            await this.modalErroExcluirCriterio.show();
        } else {
            console.error("Modal 'modalErroExcluirCriterio' não foi inicializado.");
        }
    }

    public fecharModalErro() {
        if (this.modalErroExcluirCriterio) {
            this.modalErroExcluirCriterio.hide();
        }
    }

    public async abrirModalErroInativar() {
        if (this.modalErroExcluirCriterio) {
            await this.modalErroExcluirCriterio.show();
        } else {
            console.error("Modal 'modalErroExcluirCriterio' não foi inicializado.");
        }
    }

    public fecharModalErroInativar() {
        if (this.modalErroExcluirCriterio) {
            this.modalErroExcluirCriterio.hide();
        }
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCRITERIOAUDIT: newId }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMCRITERIOAUDIT_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    /* eslint-disable */
    public canEditRow({ row }: any) {
        return row['__added_row'] === true;
    }
    /* eslint-enable */

    public salvarCriterioAuditoria({ component, event }: any) {
        event.preventDefault();
        const criterios = component.getEditedRows();

        if (criterios.length > 0) {
            if (criterios[0].NMCRITERIOAUDIT != null) {
                LoadingService.show();
                Http.post('/salvarCriteriosAvaliacao', criterios).then((response) => {
                    if (response.data.status === 'erro') {
                        this.montarModalErroInativar(response.data.relacionados);
                    } else {
                        component.datasource.get();
                        Utilities.showSuccessMessage();
                    }
                }).catch((error: any) => {
                    Utilities.showErrorMessage(error);
                }).finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
            } else {
                Utilities.alertMessage('Preencha o campo de "Nome"', 'error');
            }
        } else {
            Utilities.alertMessage('Preencha o campo de "Nome"!', 'error');
        }
    }

    public excluirCriterios({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040001CriteriosAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
    }

    public aposConfirmarDelete({ event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        const component = Metadata.getInstance<TekGrid>('tek-grid-criterios-avaliacao');
        LoadingService.show();
        Http.post('/excluirRegistros', {
            CRITERIOS: component.selectedRows.map((element: any) => element.NRCRITERIOAUDIT),
        }).then((response: IDictionary) => {
            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.nao_excluidos);
                Utilities.showSuccessMessage();
                component.datasource.get();
            } else {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public async montarModalErro(tarefas: any) {
        await this.abrirModalErro();
        const grid = Metadata.getInstance<TekGrid>('grid-conflito-criterio');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRMODELOAUDIT: element.NRMODELOAUDIT,
                NMMODELOAUDIT: element.NMMODELOAUDIT,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroInativar(tarefas: any) {
        await this.abrirModalErroInativar();
        const grid = Metadata.getInstance<TekGrid>('grid-conflito-criterio');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NRMODELOAUDIT: element.NRMODELOAUDIT,
                NMMODELOAUDIT: element.NMMODELOAUDIT,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public getParametrizacao() {
        Http.get('/getParametrizacaoCriterios').then((response: any) => {
            this.parametrizacaoPercentual = (response.data[0].IDUTILIZAPERCENTUALAUDIT === 'S');
            this.parametrizacaoNota = (response.data[0].IDUTILIZANOTAAUDIT === 'S');
        });
    }
}
