/* eslint-disable */
import { Utilities } from '@/util/Utilities';
import {
    DialogService, Form, IterableComponentRender,
    LoadingService, Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata, Loader,
    MemoryDatasource,
} from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { TekIterableComponentRender } from '@zeedhi/teknisa-components-vuetify';
import { ZdText } from '@zeedhi/vuetify';

export class Fm00040004ModeloAuditoriaController {
    public AuditoriaController = Loader.getInstance('Fm00040000AuditoriaController');

    public modalCriacaoModelo!: Modal;

    public modalErroExclusão!: Modal;

    public modalErroDados!: Modal;

    public modalErroInatividade!: Modal;

    public ultAtividadeIsAtivo!: Boolean;

    private NRMODELOAUDIT: any = 0;

    public cod: any = 0;

    public async criarModais() {
        this.modalCriacaoModelo = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00040_000_auditoria/modal/fm_00040_004_construir_modelo' });
        this.modalErroExclusão = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_exclusao_modelo', true);
        this.modalErroInatividade = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_inativa_modelo', true);
        this.modalErroDados = await ModalService.createFromJson('fm_00040_000_auditoria/modal/modal_exclusao_dados_modelo', true);
    }

    public destruirModais() {
        this.modalCriacaoModelo?.destroy();
    }

    public abrirModal({ component }: any) {
        this.modalCriacaoModelo.show();
        const nr = component.name.split('_')[1];
        this.NRMODELOAUDIT = nr;

        setTimeout(() => {
            Metadata.getInstance<Form>('form-construir-modelo').clearForm();
        }, 200);
    }

    public fecharModal() {
        this.modalCriacaoModelo.hide();
    }

    public async abrirModalErro() {
        if (this.modalErroExclusão) {
            await this.modalErroExclusão.show();
        }
    }

    public fecharModalErro() {
        if (this.modalErroExclusão) {
            this.modalErroExclusão.hide();
        }
    }

    public async abrirModalErroInativa() {
        if (this.modalErroInatividade) {
            await this.modalErroInatividade.show();
        }
    }

    public fecharModalErroInativa() {
        if (this.modalErroInatividade) {
            this.modalErroInatividade.hide();
        }
    }

    public async abrirModalErroDados() {
        if (this.modalErroDados) {
            await this.modalErroDados.show();
        }
    }

    public fecharModalErroDados() {
        if (this.modalErroDados) {
            this.modalErroDados.hide();
        }
    }

    public setFilterEstruturaModelo({ component }: IEventParam<TekGrid>) {
        component.datasource.setFilter({
            NRMODELOAUDIT: component.name.split('_')[1],
        });
    }

    public salvarModeloAuditoria() {
        const iterable = Metadata.getInstance<TekIterableComponentRender>('iterable-modelo-auditoria');

        LoadingService.show();
        Http.post('/salvarModeloAuditoria', {
            NMMODELOAUDIT: Metadata.getInstance<TextInput>('NMMODELOAUDIT').value,
        }).then(() => {
            iterable.datasource.get();
            Utilities.showSuccessMessage();
            this.AuditoriaController.fecharModalInserirModelo();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public changeTipoModelo({ component }: IEventParam<Select>) {
        const categoria = Metadata.getInstance<Select>('selecionar-categoria');
        const topicos = Metadata.getInstance<SelectMultiple>('selecionar-topicos');
        const criterios = Metadata.getInstance<SelectMultiple>('selecionar-criterios');

        if (component.value === 'C') {
            categoria.disabled = false;
            topicos.disabled = false;
            criterios.disabled = false;
            categoria.addValidation('required', {});
            topicos.removeValidation('required');
            criterios.addValidation('required', {});
        } else if (component.value === 'T') {
            categoria.disabled = false;
            topicos.disabled = false;
            criterios.disabled = true;
            categoria.addValidation('required', {});
            topicos.addValidation('required', {});
            criterios.removeValidation('required');
        }
    }

    public construirModelo() {
        const treeGrid = Metadata.getInstance<TekIterableComponentRender>('iterable-modelo-auditoria');

        Http.post('/salvarModelo', {
            NRCATEGORIAAUDIT: Metadata.getInstance<Select>('selecionar-categoria').value,
            NRMODELOAUDIT: this.NRMODELOAUDIT,
            NRTOPICOAUDIT: Metadata.getInstance<SelectMultiple>('selecionar-topicos').value,
            NRCRITERIOAUDIT: Metadata.getInstance<SelectMultiple>('selecionar-criterios').value,
        }).then(() => {
            treeGrid.datasource.get();
            Utilities.showSuccessMessage();
            this.fecharModal();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public montarIdAtivo({ component }: any) {
        if(component.text === 'S') {
            component.text = "(Ativo)";
            component.cssStyle = "font-size: 0.8375rem;";
        } else {
            component.text = "(Inativo)";
            component.cssStyle = "font-size: 0.8375rem;";
        }
    }

    public montarBtnAtivo({ component }: any) {
        const ativo = Metadata.getInstance<ZdText>(`ativo-modelo_${component.name.split('_').pop()}`);
        if(ativo.text ===  '(Ativo)') {
            component.label = "Inativar";
        } else {
            component.label = "Ativar";
        }
    }

    private codigo: any = 0;

    public excluirModelo({ event, component }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040004ModeloAuditoriaController.aposConfirmarDelete', 'Excluir Registro(s)?');
        const cod = component.name.split('_')[1];
        this.codigo = cod;
    }

    public aposConfirmarDelete() {
        const component = Metadata.getInstance<IterableComponentRender>('iterable-modelo-auditoria');

        LoadingService.show();
        Http.post('/excluirRegistros', {
            MODELO: this.codigo,
        }).then((response: IDictionary) => {
            if (response.data.status === 'erro') {
                this.montarModalErro(response.data.relacionados);
            } else { 
                Utilities.showSuccessMessage();
            }
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public async montarModalErro(relacionados: any) {
        await this.abrirModalErro();
        const grid = Metadata.getInstance<TekGrid>('grid-conflito-modelo');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        relacionados.forEach((element: any) => {
            const dados: any = {
                NMAUDITORIA: element.NMAUDITORIA,
                NRAUDITORIA: element.NRAUDITORIA,
                IDSITUACAOAUDIT: element.IDSITUACAOAUDIT,
                DTHORAAUDIT: element.DTHORAAUDIT,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroInatividade(tarefas: any) {
        await this.abrirModalErroInativa();
        const grid = Metadata.getInstance<TekGrid>('grid-conflito-inativa-model');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NMAUDITORIA: element.NMAUDITORIA,
                NRAUDITORIA: element.NRAUDITORIA,
                IDSITUACAOAUDIT: element.IDSITUACAOAUDIT,
                DTHORAAUDIT: element.DTHORAAUDIT,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public async montarModalErroDados(tarefas: any) {
        await this.abrirModalErroDados();
        const grid = Metadata.getInstance<TekGrid>('grid-conflito-dados');
        grid.datasource.data = [];
        (grid.datasource as MemoryDatasource).allData = [];

        tarefas.forEach((element: any) => {
            const dados: any = {
                NMAUDITORIA: element.NMAUDITORIA,
                NRAUDITORIA: element.NRAUDITORIA,
                IDSITUACAOAUDIT: element.IDSITUACAOAUDIT,
                DTHORAAUDIT: element.DTHORAAUDIT,
            };
            grid.datasource.post(dados);
        });
        grid.datasource.get();
    }

    public inativarModelo({ event, component }: any) {
        this.cod = component.name.split('_')[1];
        this.ultAtividadeIsAtivo = (component.label === 'Ativar');
        Utilities.showConfirmDeleteMessage(event, 'Fm00040004ModeloAuditoriaController.aposConfirmarInativar', `${component.label} Registro`, `Deseddja realmente ${component.label} o registro selecionado?`);
    }

    public aposConfirmarInativar() {
        const component = Metadata.getInstance<IterableComponentRender>('iterable-modelo-auditoria');

        LoadingService.show();
        Http.post('/inativarModelo', {
            ATIVAR: this.ultAtividadeIsAtivo,
            MODELO: this.codigo,
            NRMODELOAUDIT: this.cod,
        }).then((response: IDictionary) => {
            if (response.data.status === 'erro') {
               this.montarModalErroInatividade(response.data.relacionados);
            } else {
                Utilities.showSuccessMessage();
            }
            component.datasource.get();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public excluirDados({ event, component }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Fm00040004ModeloAuditoriaController.aposConfirmarDeleteDados', 'Excluir Registro(s)?');
        const cod = component.name.split('_')[1];
        this.codigo = cod;
    }

    public aposConfirmarDeleteDados() {
        const component = Metadata.getInstance<TekGrid>(`tek-tree-grid_${this.codigo}`);

        LoadingService.show();
        Http.post('/excluirRegistros', {
            DADOS: component.selectedRows.map((element: any) => element.CODIGO),
            MODELO: this.codigo,
        }).then((response: IDictionary) => {
            if (response.data.status === 'erro') {
                this.montarModalErroDados(response.data.relacionados);
            } else {
                Utilities.showSuccessMessage();
                component.datasource.get();
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            component.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
